"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ExportContentEntriesController = void 0;
var _uniqid = _interopRequireDefault(require("uniqid"));
var _tasks = require("@webiny/tasks");
var _types = require("../../types");
var _ExportContentEntriesController = require("./abstractions/ExportContentEntriesController");
var _constants = require("../constants");
var _getBackOffSeconds = require("../utils/helpers/getBackOffSeconds");
const prepareExportModel = model => {
  return {
    modelId: model.modelId,
    fields: model.fields
  };
};
class ExportContentEntriesController {
  async run(params) {
    const {
      context,
      response,
      input,
      store,
      trigger
    } = params;
    const {
      state,
      modelId
    } = input;
    let model;
    try {
      model = await context.cms.getModel(modelId);
    } catch (ex) {
      return response.error({
        message: `Model "${modelId}" not found.`,
        code: "MODEL_NOT_FOUND"
      });
    }
    const backOffSeconds = (0, _getBackOffSeconds.getBackOffSeconds)(store.getTask().iterations);
    const taskId = store.getTask().id;
    let entriesTask;

    /**
     * In case of no state yet, we will start the content entries export process.
     */
    const prefix = input.prefix || (0, _uniqid.default)(`${_constants.EXPORT_BASE_PATH}/${model.modelId}/${taskId}`);
    if (!state) {
      const task = await trigger({
        definition: _constants.EXPORT_CONTENT_ENTRIES_TASK,
        input: {
          prefix,
          exportAssets: input.exportAssets,
          modelId: model.modelId,
          limit: input.limit,
          where: input.where,
          sort: input.sort
        },
        name: `Export Content Entries ${taskId}`
      });
      return response.continue({
        ...input,
        prefix,
        contentEntriesTaskId: task.id,
        state: _ExportContentEntriesController.ExportContentEntriesControllerState.entryExport
      }, {
        seconds: backOffSeconds
      });
    }
    /**
     * If the state of the task is "entryExport", we need to check if there are any child tasks of the "Export Content Entries" task.
     * If there are, we need to wait for them to finish before we can proceed.
     * If there are no child tasks, we'll return an error.
     * If there are child tasks, but they are not finished, we'll return a "continue" response, which will make the task wait for X seconds before checking again.
     */
    //
    else if (state === _ExportContentEntriesController.ExportContentEntriesControllerState.entryExport) {
      if (!input.contentEntriesTaskId) {
        return response.error({
          message: `Missing "contentEntriesTaskId" in the input, but the input notes that the task is in "entryExport" state. This should not happen.`,
          code: "MISSING_CONTENT_ENTRIES_TASK_ID"
        });
      }
      entriesTask = await this.getEntriesTask(context, input.contentEntriesTaskId);
      if (!entriesTask) {
        return response.error({
          message: `Task "${input.contentEntriesTaskId}" not found.`,
          code: "TASK_NOT_FOUND"
        });
      }
      if (entriesTask.taskStatus == _tasks.TaskDataStatus.RUNNING || entriesTask.taskStatus === _tasks.TaskDataStatus.PENDING) {
        return response.continue(input, {
          seconds: backOffSeconds
        });
      } else if (entriesTask.taskStatus === _tasks.TaskDataStatus.FAILED) {
        return response.error({
          message: `Failed to export content entries. Task "${entriesTask.id}" failed.`,
          code: "EXPORT_ENTRIES_FAILED"
        });
      } else if (entriesTask.taskStatus === _tasks.TaskDataStatus.ABORTED) {
        return response.error({
          message: `Export content entries process was aborted. Task "${entriesTask.id}" was aborted.`,
          code: "EXPORT_ENTRIES_ABORTED"
        });
      } else if (!entriesTask.output) {
        return response.error({
          message: `No output found on task "${entriesTask.id}". Stopping export process.`,
          code: "NO_OUTPUT"
        });
      }
      /**
       * Possibly the task does not require any assets to be exported.
       */
      if (!input.exportAssets || entriesTask.output.files.length === 0) {
        const files = [];
        for (const file of entriesTask.output.files) {
          files.push({
            key: file.key,
            checksum: file.checksum,
            type: _types.CmsImportExportFileType.ENTRIES
          });
        }
        const output = {
          files,
          model: prepareExportModel(model)
        };
        return response.done("Export done, without assets.", output);
      }
      const assetTask = await trigger({
        definition: _constants.EXPORT_CONTENT_ASSETS_TASK,
        input: {
          prefix,
          modelId: model.modelId,
          limit: input.limit,
          where: input.where,
          sort: input.sort,
          entryAfter: undefined,
          fileAfter: undefined
        },
        name: `Export Content Assets ${taskId}`
      });
      return response.continue({
        ...input,
        contentAssetsTaskId: assetTask.id,
        state: _ExportContentEntriesController.ExportContentEntriesControllerState.assetsExport
      }, {
        seconds: backOffSeconds
      });
    }
    /**
     * If the state is "assetsExport", we need to check if there are any child tasks of the "Export Content Assets" task.
     * If there are, we need to wait for them to finish before we can proceed.
     * If there are no child tasks, we'll return as done.
     * If there are child tasks, but they are not finished, we'll return a "continue" response, which will make the task wait for X seconds before checking again.
     */
    //
    else if (state === _ExportContentEntriesController.ExportContentEntriesControllerState.assetsExport) {
      if (!input.contentEntriesTaskId) {
        return response.error({
          message: `Missing "contentEntriesTaskId" in the input, but the input notes that the task is in "assetsExport" state. This should not happen.`,
          code: "MISSING_CONTENT_ENTRIES_TASK_ID"
        });
      } else if (!input.contentAssetsTaskId) {
        return response.error({
          message: `Missing "contentAssetsTaskId" in the input, but the input notes that the task is in "assetsExport" state. This should not happen.`,
          code: "MISSING_CONTENT_ASSETS_TASK_ID"
        });
      }
      const assetsTask = await this.getAssetsTask(context, input.contentAssetsTaskId);
      if (!assetsTask) {
        return response.error({
          message: `Task "${input.contentAssetsTaskId}" not found.`,
          code: "TASK_NOT_FOUND"
        });
      }
      if (assetsTask.taskStatus == _tasks.TaskDataStatus.RUNNING || assetsTask.taskStatus === _tasks.TaskDataStatus.PENDING) {
        return response.continue({
          ...input
        }, {
          seconds: backOffSeconds
        });
      } else if (assetsTask.taskStatus === _tasks.TaskDataStatus.FAILED) {
        return response.error({
          message: `Failed to export content assets. Task "${assetsTask.id}" failed.`,
          code: "EXPORT_ASSETS_FAILED"
        });
      } else if (assetsTask.taskStatus === _tasks.TaskDataStatus.ABORTED) {
        return response.error({
          message: `Export content assets process was aborted. Task "${assetsTask.id}" was aborted.`,
          code: "EXPORT_ASSETS_ABORTED"
        });
      }
      entriesTask = await this.getEntriesTask(context, input.contentEntriesTaskId);
      const files = [];
      const entriesFiles = entriesTask?.output?.files || [];
      for (const file of entriesFiles) {
        files.push({
          key: file.key,
          checksum: file.checksum,
          type: _types.CmsImportExportFileType.ENTRIES
        });
      }
      const assetFiles = assetsTask.output?.files || [];
      for (const file of assetFiles) {
        files.push({
          key: file.key,
          checksum: file.checksum,
          type: _types.CmsImportExportFileType.ASSETS
        });
      }
      const output = {
        model: prepareExportModel(model),
        files
      };
      return response.done("Export done, with assets.", output);
    }
    return response.error({
      message: `Invalid state "${state}".`,
      code: "INVALID_STATE"
    });
  }
  async getEntriesTask(context, id) {
    try {
      const result = await context.tasks.getTask(id);
      if (result?.definitionId === _constants.EXPORT_CONTENT_ENTRIES_TASK) {
        return result;
      }
      return null;
    } catch (ex) {
      return null;
    }
  }
  async getAssetsTask(context, id) {
    try {
      const result = await context.tasks.getTask(id);
      if (result?.definitionId == _constants.EXPORT_CONTENT_ASSETS_TASK) {
        return result;
      }
      return null;
    } catch (ex) {
      return null;
    }
  }
}
exports.ExportContentEntriesController = ExportContentEntriesController;

//# sourceMappingURL=ExportContentEntriesController.js.map