"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ValidateImportFromUrl = void 0;
var _getImportExportFileType = require("../../utils/helpers/getImportExportFileType");
var _importPath = require("../../utils/helpers/importPath");
class ValidateImportFromUrl {
  constructor(params) {
    this.fileFetcher = params.fileFetcher;
    this.fileExists = params.fileExists;
  }
  async run(params) {
    const {
      response,
      input
    } = params;
    const {
      files = [],
      model
    } = input;
    const results = [];
    for (const target of files) {
      const {
        get,
        head,
        checksum
      } = target;
      const {
        type,
        pathname,
        error: fileTypeError
      } = (0, _getImportExportFileType.getImportExportFileType)(head);
      if (fileTypeError) {
        results.push({
          checked: true,
          get,
          head,
          key: target.key,
          checksum,
          error: {
            message: "File type not supported.",
            code: "FILE_TYPE_NOT_SUPPORTED",
            data: {
              pathname,
              type
            }
          },
          size: undefined,
          type: undefined
        });
        continue;
      }
      const {
        file,
        error
      } = await this.fileFetcher.head(head);
      if (error) {
        results.push({
          checked: true,
          get,
          head,
          error,
          key: target.key,
          checksum,
          size: undefined,
          type: undefined
        });
        continue;
      } else if (!file) {
        results.push({
          checked: true,
          get,
          head,
          key: target.key,
          checksum,
          error: {
            message: "File not found.",
            code: "FILE_NOT_FOUND",
            data: {
              url: head
            }
          },
          size: undefined,
          type: undefined
        });
        continue;
      } else if (file.checksum !== checksum) {
        results.push({
          checked: true,
          get,
          head,
          key: target.key,
          checksum,
          error: {
            message: "Checksum does not match.",
            code: "CHECKSUM_MISMATCH",
            data: {
              expected: checksum,
              received: file.checksum
            }
          },
          size: file.size,
          type
        });
        continue;
      }
      const key = (0, _importPath.prependImportPath)(target.key);
      const exists = await this.fileExists(key);
      if (exists) {
        results.push({
          checked: true,
          get,
          head,
          key: target.key,
          checksum,
          error: {
            message: "File already exists.",
            code: "FILE_ALREADY_EXISTS",
            data: {
              key
            }
          },
          size: file.size,
          type
        });
        continue;
      }
      results.push({
        checked: true,
        get,
        head,
        key: target.key,
        checksum,
        size: file.size,
        type
      });
    }
    if (results.length === 0) {
      return response.error({
        message: "No files found.",
        code: "NO_FILES_FOUND"
      });
    }
    const output = {
      files: results,
      modelId: model?.modelId
    };
    const filesWithErrors = results.filter(file => !!file.error);
    if (filesWithErrors.length > 0) {
      output.error = {
        message: "One or more files are invalid.",
        code: "INVALID_FILES",
        data: {
          files: filesWithErrors.map(file => {
            return file.key;
          })
        }
      };
    }
    return response.done(output);
  }
}
exports.ValidateImportFromUrl = ValidateImportFromUrl;

//# sourceMappingURL=ValidateImportFromUrl.js.map